@@ -44,6 +44,8 @@ knowledge of the CeCILL license and that you accept its terms.
 
 //From bpp-core:
 #include <Bpp/Utils/MapTools.h>
+#include <Bpp/Numeric/VectorTools.h>
+#include <Bpp/Numeric/Number.h>
 
 //From the STL:
 #include <map>
@@ -54,35 +56,74 @@ namespace bpp {
 /**
  * @brief General interface for storing statistical results.
  *
- * This interface will most likely be extended in the future...
- *
  * @author Julien Dutheil
  * @see MafStatistics
  */
 class MafStatisticsResult
 {
   protected:
-    mutable std::map<std::string, double> values_;
+    mutable std::map<std::string, BppNumberI*> values_;
 
   public:
     MafStatisticsResult(): values_() {}
     virtual ~MafStatisticsResult() {}
 
+    MafStatisticsResult(const MafStatisticsResult& msr): values_()
+    {
+      for (std::map<std::string, BppNumberI*>::const_iterator it = msr.values_.begin();
+          it != msr.values_.end();
+          ++it) {
+        values_[it->first] = it->second->clone();
+      }
+    }
+
   public:
-    virtual double getValue(const std::string& tag) const throw (Exception) {
-      std::map<std::string, double>::iterator it = values_.find(tag);
+    virtual const BppNumberI& getValue(const std::string& tag) const throw (Exception) {
+      std::map<std::string, BppNumberI*>::iterator it = values_.find(tag);
       if (it != values_.end())
-        return it->second;
+        return *it->second;
       else
         throw Exception("MafStatisticsResult::getValue(). No value found for tag: " + tag + ".");
     }
+
     /**
      * @brief Associate a value to a certain tag. Any existing tag will be overwritten
      *
      * @param tag The name of the value to associate.
      * @param value The value to associate to the tag.
      */
-    virtual void setValue(const std::string& tag, double value) throw (Exception) { values_[tag] = value; }
+    virtual void setValue(const std::string& tag, double value) throw (Exception) {
+      if (values_[tag]) {
+        delete values_[tag];
+      }
+      values_[tag] = new BppDouble(value);
+    }
+
+    /**
+     * @brief Associate a value to a certain tag. Any existing tag will be overwritten
+     *
+     * @param tag The name of the value to associate.
+     * @param value The value to associate to the tag.
+     */
+    virtual void setValue(const std::string& tag, int value) throw (Exception) {
+      if (values_[tag]) {
+        delete values_[tag];
+      }
+      values_[tag] = new BppInteger(value);
+    }
+
+    /**
+     * @brief Associate a value to a certain tag. Any existing tag will be overwritten
+     *
+     * @param tag The name of the value to associate.
+     * @param value The value to associate to the tag.
+     */
+    virtual void setValue(const std::string& tag, unsigned int value) throw (Exception) {
+      if (values_[tag]) {
+        delete values_[tag];
+      }
+      values_[tag] = new BppUnsignedInteger(value);
+    }
 
     /**
      * @return A boolean saying whether a value is available for the given tag.
@@ -114,22 +155,45 @@ class SimpleMafStatisticsResult:
     virtual ~SimpleMafStatisticsResult() {}
 
   public:
-    virtual double getValue() const { return values_[name_]; }
+    virtual const BppNumberI& getValue(const std::string& tag) const throw (Exception) { return SimpleMafStatisticsResult::getValue(tag); }
+    
+    virtual const BppNumberI& getValue() const { return *values_[name_]; }
 
-    /**
-     * @brief Associate a value to a certain tag. Any existing tag will be overwritten
-     *
-     * @param tag The name of the value to associate.
-     * @param value The value to associate to the tag.
-     */
     virtual void setValue(const std::string& tag, double value) throw (Exception) {
       if (tag == name_)
-        values_[tag] = value;
+        setValue(value);
       else
         throw Exception("SimpleMafStatisticsResult::setValue(). Unvalid tag name: " + tag + ".");
     }
-    
-    virtual void setValue(double value) { values_[name_] = value; }
+ 
+    virtual void setValue(const std::string& tag, int value) throw (Exception) {
+      if (tag == name_)
+        setValue(value);
+      else
+        throw Exception("SimpleMafStatisticsResult::setValue(). Unvalid tag name: " + tag + ".");
+    }
+ 
+    virtual void setValue(const std::string& tag, unsigned int value) throw (Exception) {
+      if (tag == name_)
+        setValue(value);
+      else
+        throw Exception("SimpleMafStatisticsResult::setValue(). Unvalid tag name: " + tag + ".");
+    }
+   
+    virtual void setValue(double value) {
+      if (values_[name_]) delete values_[name_];
+      values_[name_] = new BppDouble(value);
+    }
+
+    virtual void setValue(int value) {
+      if (values_[name_]) delete values_[name_];
+      values_[name_] = new BppInteger(value);
+    }
+
+    virtual void setValue(unsigned int value) {
+      if (values_[name_]) delete values_[name_];
+      values_[name_] = new BppUnsignedInteger(value);
+    }
 
 };
 
@@ -342,7 +406,7 @@ class CharacterCountsMafStatistics:
 /**
  * @brief Partial implementation of MafStatistics for method working on a subset of species, in a site-wise manner.
  *
- * This class stores a seleciton of species and create for each block the corresponding SiteContainer.
+ * This class stores a selection of species and create for each block the corresponding SiteContainer instance.
  */
 class AbstractSpeciesSelectionMafStatistics:
   public virtual MafStatistics
@@ -355,7 +419,27 @@ class AbstractSpeciesSelectionMafStatistics:
       species_(species) {}
 
   protected:
-    SiteContainer* getSiteContainer(const MafBlock& block);
+    SiteContainer* getSiteContainer_(const MafBlock& block);
+
+};
+
+
+/**
+ * @brief Partial implementation of MafStatistics for method working on multiple distinct subsets of species, in a site-wise manner.
+ *
+ * This class stores two non-overlapping selections of species and create for each block the corresponding SiteContainer instances.
+ */
+class AbstractSpeciesMultipleSelectionMafStatistics:
+  public virtual MafStatistics
+{
+  private:
+    std::vector< std::vector<std::string> > species_;
+
+  public:
+    AbstractSpeciesMultipleSelectionMafStatistics(const std::vector< std::vector<std::string> >& species);
+
+  protected:
+    std::vector<SiteContainer*> getSiteContainers_(const MafBlock& block);
 
 };
 
@@ -363,7 +447,8 @@ class AbstractSpeciesSelectionMafStatistics:
 /**
  * @brief Compute the Site Frequency Spectrum of a maf block.
  *
- * The ancestral states are considered as unknown, so that 10000 and 11110 sites are treated equally.
+ * If no outgroup is provided, the ancestral states are considered as unknown
+ * and the unfolded spectrum is computed, so that 10000 and 11110 sites are treated equally.
  */
 class SiteFrequencySpectrumMafStatistics:
   public AbstractMafStatistics,
@@ -375,7 +460,8 @@ class SiteFrequencySpectrumMafStatistics:
         std::vector<double> bounds_;
 
       public:
-        Categorizer(const std::vector<double>& bounds): bounds_(bounds) {
+        Categorizer(const std::vector<double>& bounds):
+          bounds_(bounds) {
           std::sort(bounds_.begin(), bounds_.end());
         }
 
@@ -398,14 +484,16 @@ class SiteFrequencySpectrumMafStatistics:
     const Alphabet* alphabet_;
     Categorizer categorizer_;
     std::vector<unsigned int> counts_;
+    std::string outgroup_;
 
   public:
-    SiteFrequencySpectrumMafStatistics(const Alphabet* alphabet, const std::vector<double>& bounds, const std::vector<std::string>& ingroup):
+    SiteFrequencySpectrumMafStatistics(const Alphabet* alphabet, const std::vector<double>& bounds, const std::vector<std::string>& ingroup, const std::string outgroup = ""):
       AbstractMafStatistics(),
       AbstractSpeciesSelectionMafStatistics(ingroup),
       alphabet_(alphabet),
       categorizer_(bounds),
-      counts_(bounds.size() - 1)
+      counts_(bounds.size() - 1),
+      outgroup_(outgroup)
     {}
 
     SiteFrequencySpectrumMafStatistics(const SiteFrequencySpectrumMafStatistics& stats):
@@ -413,7 +501,8 @@ class SiteFrequencySpectrumMafStatistics:
       AbstractSpeciesSelectionMafStatistics(stats),
       alphabet_(stats.alphabet_),
       categorizer_(stats.categorizer_),
-      counts_(stats.counts_)
+      counts_(stats.counts_),
+      outgroup_(stats.outgroup_)
     {}
 
     SiteFrequencySpectrumMafStatistics& operator=(const SiteFrequencySpectrumMafStatistics& stats) {
@@ -422,6 +511,7 @@ class SiteFrequencySpectrumMafStatistics:
       alphabet_    = stats.alphabet_;
       categorizer_ = stats.categorizer_;
       counts_      = stats.counts_;
+      outgroup_    = stats.outgroup_;
       return *this;
     }
 
@@ -434,6 +524,66 @@ class SiteFrequencySpectrumMafStatistics:
     std::vector<std::string> getSupportedTags() const;
 };
 
+
+/**
+ * @brief Compute the frequency of site patterns for a quadruplet of species.
+ *
+ * Only parsimony informative sites are categorized.
+ * Species: A B C D
+ * P1       1 1 0 0
+ * P2       0 1 1 0
+ * P3       1 0 1 0
+ * Sites with more than two states are ignored, as well as sites containing gaps or unresolved characters.
+ */
+class FourSpeciesPatternCountsMafStatistics:
+  public AbstractMafStatistics,
+  public AbstractSpeciesSelectionMafStatistics
+{
+  private:
+    const Alphabet* alphabet_;
+    std::vector<unsigned int> counts_;
+
+  public:
+    FourSpeciesPatternCountsMafStatistics(
+        const Alphabet* alphabet,
+        const std::vector<std::string>& species):
+      AbstractMafStatistics(),
+      AbstractSpeciesSelectionMafStatistics(species),
+      alphabet_(alphabet),
+      counts_(6)
+    {
+      if (species.size() != 4)
+        throw Exception("FourSpeciesPatternCountsMafStatistics, constructor: 4 species should be provided.");
+      if (VectorTools::unique(species).size() != 4)
+        throw Exception("FourSpeciesPatternCountsMafStatistics, constructor: duplicated species name!");
+    }
+
+    FourSpeciesPatternCountsMafStatistics(const FourSpeciesPatternCountsMafStatistics& stats):
+      AbstractMafStatistics(),
+      AbstractSpeciesSelectionMafStatistics(stats),
+      alphabet_(stats.alphabet_),
+      counts_(stats.counts_)
+    {}
+
+    FourSpeciesPatternCountsMafStatistics& operator=(const FourSpeciesPatternCountsMafStatistics& stats) {
+      AbstractMafStatistics::operator=(stats);
+      AbstractSpeciesSelectionMafStatistics::operator=(stats);
+      alphabet_    = stats.alphabet_;
+      counts_      = stats.counts_;
+      return *this;
+    }
+
+    virtual ~FourSpeciesPatternCountsMafStatistics() {}
+
+  public:
+    std::string getShortName() const { return "FourSpeciesPatternCounts"; }
+    std::string getFullName() const { return "FourSpecies pattern counts."; }
+    void compute(const MafBlock& block);
+    std::vector<std::string> getSupportedTags() const;
+};
+
+
+
 /**
  * @brief Compute a few site statistics in a maf block.
  *
@@ -461,6 +611,77 @@ class SiteMafStatistics:
     std::vector<std::string> getSupportedTags() const;
 };
 
+
+/**
+ * @brief Counts number of polymorphic / fixed sites in two populations.
+ *
+ * The two populations are defined as two distinct sets of species.
+ * The following counts are computed and returned:
+ * - P: number of sites polymorphic in both populations
+ * - F: number of sites fixed in both populations
+ * - FF: number of sites fixed in both populations, but with distinct states
+ * - PF / FP: number of sites polymorphic in one species and fixed in the other.
+ * - X: unresolved (because of gap or generic character)
+ * - FX / PX / XF / XP: unresolved in one population
+ */
+class PolymorphismMafStatistics:
+  public AbstractMafStatistics,
+  public AbstractSpeciesMultipleSelectionMafStatistics
+{
+  public:
+    PolymorphismMafStatistics(const std::vector< std::vector<std::string> >& species):
+      AbstractMafStatistics(),
+      AbstractSpeciesMultipleSelectionMafStatistics(species)
+    {
+      if (species.size() != 2)
+        throw Exception("PolymorphismStatistics: exactly two species selection should be provided.");
+    }
+
+    virtual ~PolymorphismMafStatistics() {}
+
+  public:
+    std::string getShortName() const { return "PolymorphismStatistics"; }
+    std::string getFullName() const { return "Polymorphism statistics."; }
+    void compute(const MafBlock& block);
+    std::vector<std::string> getSupportedTags() const;
+
+  private:
+    static std::vector<int> getPatterns_(const SiteContainer& sites); 
+};
+
+
+
+/**
+ * @brief Provide estimates of sequence diversity.
+ *
+ * - Number of segregating sites
+ * - Watterson's theta (per site)
+ *
+ * Only fully resolved sites are analyzed (no gap, no generic character).
+ */
+class SequenceDiversityMafStatistics:
+  public AbstractMafStatistics,
+  public AbstractSpeciesSelectionMafStatistics
+{
+  public:
+    SequenceDiversityMafStatistics(const std::vector<std::string>& ingroup):
+      AbstractMafStatistics(),
+      AbstractSpeciesSelectionMafStatistics(ingroup)
+    {}
+
+    virtual ~SequenceDiversityMafStatistics() {}
+
+  public:
+    std::string getShortName() const { return "SequenceDiversityStatistics"; }
+    std::string getFullName() const { return "Sequence diversity statistics."; }
+    void compute(const MafBlock& block);
+    std::vector<std::string> getSupportedTags() const;
+
+  private:
+    static std::vector<int> getPatterns_(const SiteContainer& sites); 
+};
+
+
 } // end of namespace bpp
 
 #endif //_MAFSTATISTICS_H_
